/**
 * Connect to the gateway with Cord.js
 * @packageDocumentation
 *
 * @remarks
 * This plugin uses {@link https://discord.js.org/ | Discord.js} under the hood.
 */

import { ClientEvents } from 'discord.js';
import { ClientOptions } from 'discord.js';
import { Context } from '@cordjs/client';
import { PluginFactory } from '@cordjs/client';
import { PluginInstance } from '@cordjs/client';
import { PluginInterface } from '@cordjs/client';

/**
 * The gateway plugin
 * @public
 *
 * @param options
 */
declare const Gateway: (options: GatewayOptions) => PluginFactory;
export { Gateway }
export default Gateway;

/**
 * The context
 */
export declare class GatewayContext<K extends keyof ClientEvents> extends Context {
    data: ClientEvents[K];
    constructor(path: string[], data: ClientEvents[K]);
}

/**
 * A plugin instance
 * @public
 */
export declare class GatewayInstance extends PluginInstance {
    /**
     * The middleware name
     */
    middlewareName: string;
    constructor(options: PluginInterface, middlewareName: string);
}

/**
 * {@link Gateway} options
 * @public
 */
export declare interface GatewayOptions {
    /**
     * The token to login with
     */
    token: string;
    /**
     * The middleware name
     *
     * @defaultValue 'gateway'
     */
    middlewareName?: string;
    /**
     * The {@link https://discord.js.org/#/docs/main/stable/typedef/ClientOptions | Discord.js client options}
     */
    client: ClientOptions;
    /**
     * Whether or not to catch all events
     *
     * @remarks
     * This will monkey patch the `emit` function
     *
     * @defaultValue false
     */
    catchAll?: boolean;
}

export { }
