/**
 * Connect to the gateway with Cord.js
 * @packageDocumentation
 *
 * @remarks
 * This plugin uses {@link https://discord.js.org/ | Discord.js} under the hood.
 */
import { Context, PluginFactory, PluginInstance, PluginInterface } from '@cordjs/client';
import { ClientEvents, ClientOptions } from 'discord.js';
/**
 * {@link Gateway} options
 * @public
 */
export interface GatewayOptions {
    /**
     * The token to login with
     */
    token: string;
    /**
     * The middleware name
     *
     * @defaultValue 'gateway'
     */
    middlewareName?: string;
    /**
     * The {@link https://discord.js.org/#/docs/main/stable/typedef/ClientOptions | Discord.js client options}
     */
    client: ClientOptions;
    /**
     * Whether or not to catch all events
     *
     * @remarks
     * This will monkey patch the `emit` function
     *
     * @defaultValue false
     */
    catchAll?: boolean;
}
/**
 * A plugin instance
 * @public
 */
export declare class GatewayInstance extends PluginInstance {
    /**
     * The middleware name
     */
    middlewareName: string;
    constructor(options: PluginInterface, middlewareName: string);
}
/**
 * The context
 */
export declare class GatewayContext<K extends keyof ClientEvents> extends Context {
    data: ClientEvents[K];
    constructor(path: string[], data: ClientEvents[K]);
}
/**
 * The gateway plugin
 * @public
 *
 * @param options
 */
export declare const Gateway: (options: GatewayOptions) => PluginFactory;
declare module '@cordjs/client' {
    interface Client {
        gateway: MiddlewareGroup<{
            [K in keyof ClientEvents]: Middleware<GatewayContext<K>>;
        }>;
    }
}
export default Gateway;
//# sourceMappingURL=index.d.ts.map