"use strict";
/**
 * Connect to the gateway with Cord.js
 * @packageDocumentation
 *
 * @remarks
 * This plugin uses {@link https://discord.js.org/ | Discord.js} under the hood.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Gateway = exports.GatewayContext = exports.GatewayInstance = void 0;
const client_1 = require("@cordjs/client");
const discord_js_1 = require("discord.js");
/**
 * A plugin instance
 * @public
 */
class GatewayInstance extends client_1.PluginInstance {
    constructor(options, middlewareName) {
        super(options);
        this.middlewareName = middlewareName;
    }
}
exports.GatewayInstance = GatewayInstance;
/**
 * The context
 */
class GatewayContext extends client_1.Context {
    constructor(path, data) {
        super(path);
        this.data = data;
    }
}
exports.GatewayContext = GatewayContext;
/**
 * The gateway plugin
 * @public
 *
 * @param options
 */
const Gateway = (options) => (client, actions) => {
    const { defineMiddlewareRoot } = actions;
    const { middlewareName = 'gateway', client: clientOptions, token, catchAll, } = options;
    const djsClient = new discord_js_1.Client(clientOptions);
    defineMiddlewareRoot(middlewareName !== null && middlewareName !== void 0 ? middlewareName : 'gateway');
    function addEventListeners() {
        if (catchAll) {
            const emit = djsClient.emit;
            djsClient.emit = (eventName, ...args) => {
                client.executeMiddleware(new GatewayContext([middlewareName, eventName], args));
                return emit.apply(djsClient, [eventName, args]);
            };
            return;
        }
        const added = {};
        for (const middleware of client.middleware) {
            if (middleware.path.length === 2 &&
                middleware.path[0] === middlewareName) {
                if (!(middleware.path[1] in added)) {
                    const eventName = middleware.path[1];
                    added[eventName] = true;
                    djsClient.on(eventName, (...args) => {
                        client.executeMiddleware(new GatewayContext([middlewareName, eventName], args));
                    });
                }
            }
        }
    }
    return {
        id: '@cordjs/gateway',
        instance: options => new GatewayInstance(options, middlewareName),
        start() {
            return __awaiter(this, void 0, void 0, function* () {
                addEventListeners();
                yield djsClient.login(token);
            });
        },
    };
};
exports.Gateway = Gateway;
exports.default = exports.Gateway;
//# sourceMappingURL=index.js.map